package fr.mrdev023.vulkan_java.vk;

import org.lwjgl.PointerBuffer;
import org.lwjgl.system.MemoryStack;
import org.lwjgl.vulkan.*;
import org.tinylog.Logger;

import static org.lwjgl.vulkan.VK11.*;

public class Queue {

    private final VkQueue vkQueue;

    public Queue(Device device, int queueFamilyIndex, int queueIndex) {
        Logger.debug("Creating queue");

        try (MemoryStack stack = MemoryStack.stackPush()) {
            PointerBuffer pQueue = stack.mallocPointer(1);
            vkGetDeviceQueue(device.getVkDevice(), queueFamilyIndex, queueIndex, pQueue);
            long queue = pQueue.get(0);
            vkQueue = new VkQueue(queue, device.getVkDevice());
        }
    }

    public VkQueue getVkQueue() {
        return vkQueue;
    }

    public void waitIdle() {
        vkQueueWaitIdle(vkQueue);
    }

    public static class GraphicsQueue extends Queue {
        public GraphicsQueue(Device device, int queueFamilyIndex, int queueIndex) {
            super(device, queueFamilyIndex, queueIndex);
        }
    }

    public static class ComputeQueue extends Queue {
        public ComputeQueue(Device device, int queueFamilyIndex, int queueIndex) {
            super(device, queueFamilyIndex, queueIndex);
        }
    }

    public static class TransferQueue extends Queue {
        public TransferQueue(Device device, int queueFamilyIndex, int queueIndex) {
            super(device, queueFamilyIndex, queueIndex);
        }
    }
}