package fr.mrdev023.vulkan_java.vk;

import fr.mrdev023.vulkan_java.vk.utils.SuitablePhysicalDeviceFinder;
import fr.mrdev023.vulkan_java.window.Display;

import java.util.Set;

import org.lwjgl.vulkan.KHRDynamicRendering;

import static org.lwjgl.glfw.GLFWVulkan.glfwVulkanSupported;

public class Vulkan {

    private static Instance instance;
    private static PhysicalDevice physicalDevice;
    private static Device device;
    private static Surface surface;

    public static void init() throws VulkanError {
        if (!glfwVulkanSupported()) {
            throw new IllegalStateException("Cannot find a compatible Vulkan installable client driver (ICD)");
        }

        instance = new Instance(true);
        surface = new Surface(instance, Display.getWindow());

        var criteria = new SuitablePhysicalDeviceFinder.Criteria()
                .withGraphicsQueue(true)
                .withComputeQueue(true)
                .withTransferQueue(true)
                .withSurfaceSupport(surface)
                .withExtensions(Set.of(KHRDynamicRendering.VK_KHR_DYNAMIC_RENDERING_EXTENSION_NAME));

        var physicalDeviceMatch = SuitablePhysicalDeviceFinder.findBestPhysicalDevice(instance, criteria);
        if (physicalDeviceMatch == null) {
            throw new RuntimeException("No suitable physical device found");
        }

        physicalDevice = physicalDeviceMatch.physicalDevice;
        device = new Device(physicalDeviceMatch);
    }

    public static void destroy() {
        surface.destroy();
        device.destroy();
        physicalDevice.destroy();
        instance.destroy();
    }

}
